#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.

import os
import unittest

from oc_ocdm.counter_handler.sqlite_counter_handler import SqliteCounterHandler
from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.prov.entities.snapshot_entity import SnapshotEntity
from oc_ocdm.prov.prov_set import ProvSet
from oc_ocdm.reader import Reader
from oc_ocdm.storer import Storer
from rdflib import URIRef


class TestProvSet(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    def setUp(self):
        self.graph_set = GraphSet("http://test/", "./info_dir/", "", False)
        counter_db_path = 'oc_ocdm/test/prov/prov_counter.db'
        if os.path.exists(counter_db_path):
            os.remove(counter_db_path)
        self.prov_set = ProvSet(self.graph_set, "http://test/", "./info_dir/", False, custom_counter_handler=SqliteCounterHandler(counter_db_path), supplier_prefix="")
        self.cur_time = 1607375859.846196
        self.cur_time_str = '2020-12-07T21:17:39+00:00'

    def test_add_se(self):
        prov_subj = self.graph_set.add_br(self.resp_agent)
        se = self.prov_set.add_se(prov_subj)

        self.assertIsNotNone(se)
        self.assertIsInstance(se, SnapshotEntity)
        self.assertEqual(str(se.g.identifier), str(prov_subj.res) + "/prov/")

    def test_creation_merged_entity(self):
        a = self.graph_set.add_br(self.resp_agent)
        b = self.graph_set.add_br(self.resp_agent)
        a.merge(b, prefer_self=True)

        result = self.prov_set.generate_provenance(self.cur_time)
        print(a.res, b.res, URIRef(a.res + '/prov/se/1'), self.prov_set.get_entity(URIRef(a.res + '/prov/se/1')), self.prov_set.res_to_entity)
        se_a = self.prov_set.get_entity(URIRef(a.res + '/prov/se/1'))
        self.assertIsNotNone(se_a)
        self.assertIsInstance(se_a, SnapshotEntity)
        self.assertEqual(a.res, se_a.get_is_snapshot_of())
        self.assertEqual(self.cur_time_str, se_a.get_generation_time())
        if a.source is not None:
            self.assertEqual(a.source, str(se_a.get_primary_source()))
        if a.resp_agent is not None:
            self.assertEqual(a.resp_agent, str(se_a.get_resp_agent()))
        self.assertEqual(f"The entity '{a.res}' has been created.", se_a.get_description())

    def test_no_snapshot_merged_entity(self):
        a = self.graph_set.add_br(self.resp_agent)
        b = self.graph_set.add_br(self.resp_agent)
        a.merge(b)
        se_a_1 = self.prov_set.add_se(a)

        # This avoids that the presence of the mandatory rdf:type gets interpreted
        # as a modification with respect to an empty preexisting_graph:
        a.remove_every_triple()

        result = self.prov_set.generate_provenance(self.cur_time)
        
        se_a_2 = self.prov_set.get_entity(URIRef(a.res + '/prov/se/2'))
        self.assertIsNone(se_a_2)

    def test_modification_merged_entity(self):
        title = "TEST TITLE"
        a = self.graph_set.add_br(self.resp_agent)
        b = self.graph_set.add_br(self.resp_agent)
        b.has_title(title)
        a.merge(b)
        se_a_1 = self.prov_set.add_se(a)

        result = self.prov_set.generate_provenance(self.cur_time)
        
        self.assertEqual(self.cur_time_str, se_a_1.get_invalidation_time())

        se_a_2 = self.prov_set.get_entity(URIRef(a.res + '/prov/se/2'))
        self.assertIsNotNone(se_a_2)
        self.assertIsInstance(se_a_2, SnapshotEntity)
        self.assertEqual(a.res, se_a_2.get_is_snapshot_of())
        self.assertEqual(self.cur_time_str, se_a_2.get_generation_time())
        if a.source is not None:
            self.assertEqual(a.source, str(se_a_2.get_primary_source()))
        if a.resp_agent is not None:
            self.assertEqual(a.resp_agent, str(se_a_2.get_resp_agent()))
        self.assertSetEqual({se_a_1}, set(se_a_2.get_derives_from()))
        self.assertIsNotNone(se_a_2.get_update_action())
        self.assertEqual(f"The entity '{a.res}' has been modified.", se_a_2.get_description())

    def test_merge_merged_entity(self):
        a = self.graph_set.add_br(self.resp_agent)
        b = self.graph_set.add_br(self.resp_agent)
        c = self.graph_set.add_br(self.resp_agent)

        se_a_1 = self.prov_set.add_se(a)
        se_a_1.has_generation_time("2020-01-01T00:00:00+00:00")
        se_a_1.has_primary_source(URIRef("http://example.org/source_a"))
        
        se_b_1 = self.prov_set.add_se(b)
        se_b_1.has_generation_time("2020-02-01T00:00:00+00:00")
        se_b_1.has_primary_source(URIRef("http://example.org/source_b"))
        
        se_c_1 = self.prov_set.add_se(c)
        se_c_1.has_generation_time("2020-03-01T00:00:00+00:00")
        se_c_1.has_primary_source(URIRef("http://example.org/source_c"))

        a.merge(b)
        a.merge(c)

        result = self.prov_set.generate_provenance(self.cur_time)

        se_a_2 = self.prov_set.get_entity(URIRef(a.res + '/prov/se/2'))
        self.assertIsNotNone(se_a_2)
        self.assertIsInstance(se_a_2, SnapshotEntity)
        self.assertEqual(a.res, se_a_2.get_is_snapshot_of())
        self.assertEqual(self.cur_time_str, se_a_2.get_generation_time())
        if a.source is not None:
            self.assertEqual(a.source, str(se_a_2.get_primary_source()))
        if a.resp_agent is not None:
            self.assertEqual(a.resp_agent, str(se_a_2.get_resp_agent()))
        self.assertNotEqual("2020-01-01T00:00:00+00:00", se_a_2.get_generation_time())
        self.assertNotEqual(URIRef("http://example.org/source_a"), se_a_2.get_primary_source())
        self.assertEqual(self.cur_time_str, se_a_2.get_generation_time())
        if a.source is not None:
            self.assertEqual(a.source, str(se_a_2.get_primary_source()))
        self.assertSetEqual({se_a_1, se_b_1, se_c_1}, set(se_a_2.get_derives_from()))
        self.assertEqual(self.cur_time_str, se_a_1.get_invalidation_time())
        self.assertEqual(f"The entity '{a.res}' has been merged with '{b.res}', '{c.res}'.", se_a_2.get_description())

    def test_creation_non_merged_entity(self):
        a = self.graph_set.add_br(self.resp_agent)

        result = self.prov_set.generate_provenance(self.cur_time)
        
        se_a = self.prov_set.get_entity(URIRef(a.res + '/prov/se/1'))
        self.assertIsNotNone(se_a)
        self.assertIsInstance(se_a, SnapshotEntity)
        self.assertEqual(a.res, se_a.get_is_snapshot_of())
        self.assertEqual(self.cur_time_str, se_a.get_generation_time())
        if a.source is not None:
            self.assertEqual(a.source, str(se_a.get_primary_source()))
        if a.resp_agent is not None:
            self.assertEqual(a.resp_agent, str(se_a.get_resp_agent()))
        self.assertEqual(f"The entity '{a.res}' has been created.", se_a.get_description())

    def test_no_snapshot_non_merged_entity_scenario1(self):
        a = self.graph_set.add_br(self.resp_agent)
        a.mark_as_to_be_deleted()

        result = self.prov_set.generate_provenance(self.cur_time)
        
        se_a = self.prov_set.get_entity(URIRef(a.res + '/prov/se/1'))
        self.assertIsNone(se_a)

    def test_no_snapshot_merged_entity_scenario2(self):
        a = self.graph_set.add_br(self.resp_agent)
        se_a_1 = self.prov_set.add_se(a)

        # This avoids that the presence of the mandatory rdf:type gets interpreted
        # as a modification with respect to an empty preexisting_graph:
        a.remove_every_triple()

        result = self.prov_set.generate_provenance(self.cur_time)
        
        se_a_2 = self.prov_set.get_entity(URIRef(a.res + '/prov/se/2'))
        self.assertIsNone(se_a_2)

    def test_deletion_non_merged_entity(self):
        title = "TEST TITLE"
        a = self.graph_set.add_br(self.resp_agent)
        se_a_1 = self.prov_set.add_se(a)
        a.has_title(title)
        a.mark_as_to_be_deleted()

        result = self.prov_set.generate_provenance(self.cur_time)
        
        self.assertEqual(self.cur_time_str, se_a_1.get_invalidation_time())

        se_a_2 = self.prov_set.get_entity(URIRef(a.res + '/prov/se/2'))
        self.assertIsNotNone(se_a_2)
        self.assertIsInstance(se_a_2, SnapshotEntity)
        self.assertEqual(a.res, se_a_2.get_is_snapshot_of())
        self.assertEqual(self.cur_time_str, se_a_2.get_generation_time())
        if a.source is not None:
            self.assertEqual(a.source, str(se_a_2.get_primary_source()))
        if a.resp_agent is not None:
            self.assertEqual(a.resp_agent, str(se_a_2.get_resp_agent()))
        self.assertSetEqual({se_a_1}, set(se_a_2.get_derives_from()))
        self.assertEqual(f"The entity '{a.res}' has been deleted.", se_a_2.get_description())

    def test_modification_non_merged_entity(self):
        title = "TEST TITLE"
        a = self.graph_set.add_br(self.resp_agent)
        se_a_1 = self.prov_set.add_se(a)
        a.has_title(title)

        result = self.prov_set.generate_provenance(self.cur_time)
        
        self.assertEqual(self.cur_time_str, se_a_1.get_invalidation_time())

        se_a_2 = self.prov_set.get_entity(URIRef(a.res + '/prov/se/2'))
        self.assertIsNotNone(se_a_2)
        self.assertIsInstance(se_a_2, SnapshotEntity)
        self.assertEqual(a.res, se_a_2.get_is_snapshot_of())
        self.assertEqual(self.cur_time_str, se_a_2.get_generation_time())
        if a.source is not None:
            self.assertEqual(a.source, str(se_a_2.get_primary_source()))
        if a.resp_agent is not None:
            self.assertEqual(a.resp_agent, str(se_a_2.get_resp_agent()))
        self.assertSetEqual({se_a_1}, set(se_a_2.get_derives_from()))
        self.assertIsNotNone(se_a_2.get_update_action())
        self.assertEqual(f"The entity '{a.res}' has been modified.", se_a_2.get_description())

    def test_retrieve_last_snapshot(self):
        br = self.graph_set.add_br(self.resp_agent)
        br_res = br.res
        result = self.prov_set._retrieve_last_snapshot(br_res)
        self.assertIsNone(result)

        se = self.prov_set.add_se(br)
        result = self.prov_set._retrieve_last_snapshot(br_res)
        self.assertIsNotNone(result)
        self.assertEqual(str(result), str(se))

        prov_subject = URIRef('https://w3id.org/oc/corpus/br/0')
        self.assertRaises(ValueError, self.prov_set._retrieve_last_snapshot, prov_subject)

        prov_subject = URIRef('https://w3id.org/oc/corpus/br/-1')
        self.assertRaises(ValueError, self.prov_set._retrieve_last_snapshot, prov_subject)

        prov_subject = URIRef('https://w3id.org/oc/corpus/br/abc')
        self.assertRaises(ValueError, self.prov_set._retrieve_last_snapshot, prov_subject)

class TestProvSetWorkflow(unittest.TestCase):
    def setUp(self):
        self.test_dir = os.path.join('oc_ocdm', 'test', 'prov', 'provset_workflow_data') + os.sep
        if not os.path.exists(self.test_dir):
            os.makedirs(self.test_dir)
        self.base_iri = "http://test/"
        self.resp_agent = 'http://resp_agent.test/'

    def tearDown(self):
        import shutil
        shutil.rmtree(self.test_dir)

    def test_full_workflow(self):
        # Step 1: Create initial data
        graph_set = GraphSet(self.base_iri, self.test_dir, supplier_prefix="060")
        prov_set = ProvSet(graph_set, self.base_iri, self.test_dir, supplier_prefix="060")

        a = graph_set.add_br(self.resp_agent)
        b = graph_set.add_br(self.resp_agent)
        c = graph_set.add_br(self.resp_agent)

        # Create initial snapshots with specific metadata
        se_a = prov_set.add_se(a)
        se_a.has_generation_time("2020-01-01T00:00:00Z")
        se_a.has_primary_source(URIRef("http://example.org/source_a"))

        se_b = prov_set.add_se(b)
        se_b.has_generation_time("2020-02-01T00:00:00Z")
        se_b.has_primary_source(URIRef("http://example.org/source_b"))

        se_c = prov_set.add_se(c)
        se_c.has_generation_time("2020-03-01T00:00:00Z")
        se_c.has_primary_source(URIRef("http://example.org/source_c"))

        # Step 2: Save the data
        storer = Storer(graph_set)
        storer.store_all(self.test_dir, self.base_iri)

        prov_storer = Storer(prov_set)
        prov_storer.store_all(self.test_dir, self.base_iri)

        graph_set.commit_changes()

        # Step 3: Create a new GraphSet and ProvSet, and load the saved data
        new_graph_set = GraphSet(self.base_iri, self.test_dir, supplier_prefix="")
        new_prov_set = ProvSet(new_graph_set, self.base_iri, self.test_dir, supplier_prefix="")

        reader = Reader()
        for dirpath, dirnames, filenames in os.walk(self.test_dir):
            for filename in filenames:
                if filename.endswith('.json') and not dirpath.endswith('prov'):
                    full_path = os.path.join(dirpath, filename)
                    loaded_graph = reader.load(full_path)
                    reader.import_entities_from_graph(new_graph_set, results=loaded_graph, resp_agent=self.resp_agent)

        # Step 4: Perform merge operation
        new_a = new_graph_set.get_entity(a.res)
        new_b = new_graph_set.get_entity(b.res)
        new_c = new_graph_set.get_entity(c.res)

        new_a.merge(new_b)
        new_a.merge(new_c)

        # Step 5: Generate new provenance
        cur_time = 1585692000
        new_prov_set.generate_provenance(cur_time)

        # Step 6: Save the updated data
        new_storer = Storer(new_graph_set)
        new_storer.store_all(self.test_dir, self.base_iri)

        new_prov_storer = Storer(new_prov_set)
        new_prov_storer.store_all(self.test_dir, self.base_iri)

        # Step 7: Load and check the final state
        final_graph_set = GraphSet(self.base_iri, self.test_dir, supplier_prefix="")
        final_prov_set = ProvSet(final_graph_set, self.base_iri, self.test_dir, supplier_prefix="")

        final_reader = Reader()
        for dirpath, dirnames, filenames in os.walk(self.test_dir):
            for filename in filenames:
                if filename.endswith('.json') and not dirpath.endswith('prov'):
                    full_path = os.path.join(dirpath, filename)
                    loaded_graph = reader.load(full_path)
                    final_reader.import_entities_from_graph(final_graph_set, results=loaded_graph, resp_agent=self.resp_agent)

        # Check the final state
        final_a = final_graph_set.get_entity(a.res)
        final_se_a = new_prov_set.get_entity(URIRef(a.res + '/prov/se/2'))

        self.assertIsNotNone(final_se_a)
        # self.assertEqual(cur_time, final_se_a.get_generation_time())
        # self.assertNotEqual("2020-01-01T00:00:00Z", final_se_a.get_generation_time())
        # self.assertNotEqual(URIRef("http://example.org/source_a"), final_se_a.get_primary_source())

        # # Check that it derives from all previous snapshots
        # derived_from = set(final_se_a.get_derives_from())
        # self.assertEqual(3, len(derived_from))
        # self.assertTrue(all(se.res in derived_from for se in [se_a, se_b, se_c]))

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from oc_ocdm.metadata.metadata_entity import MetadataEntity
from oc_ocdm.metadata.metadata_set import MetadataSet
from rdflib import XSD, Literal, URIRef


class TestDataset(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.metadata_set = MetadataSet("http://test/", "./info_dir/", False)

    def setUp(self):
        self.dataset = self.metadata_set.add_dataset("ocdmTest", self.resp_agent)
        self.sub_dataset = self.metadata_set.add_dataset("subDataset", self.resp_agent)
        self.di = self.metadata_set.add_di("ocdmTest", self.resp_agent)

    def test_has_title(self):
        title = "Resource"
        result = self.dataset.has_title(title)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_title, Literal(title, datatype=XSD.string)
        self.assertIn(triple, self.dataset.g)

    def test_has_description(self):
        description = "Resource"
        result = self.dataset.has_description(description)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_description, Literal(description, datatype=XSD.string)
        self.assertIn(triple, self.dataset.g)

    def test_has_publication_date(self):
        string = "2020-05-25T12:12:00"
        result = self.dataset.has_publication_date(string)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_issued, Literal(string, datatype=XSD.dateTime,
                                                                      normalize=False)
        self.assertIn(triple, self.dataset.g)

    def test_has_modification_date(self):
        string = "2020-05-25T12:12:00"
        result = self.dataset.has_modification_date(string)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_modified, Literal(string, datatype=XSD.dateTime,
                                                                        normalize=False)
        self.assertIn(triple, self.dataset.g)

    def test_has_keyword(self):
        keyword = "Resource"
        result = self.dataset.has_keyword(keyword)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_keyword, Literal(keyword, datatype=XSD.string)
        self.assertIn(triple, self.dataset.g)

    def test_has_subject(self):
        subject = URIRef("http://subject/")
        result = self.dataset.has_subject(subject)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_subject, subject
        self.assertIn(triple, self.dataset.g)

    def test_has_landing_page(self):
        page = URIRef("http://landing.page/")
        result = self.dataset.has_landing_page(page)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_landing_page, page
        self.assertIn(triple, self.dataset.g)

    def test_has_sub_dataset(self):
        result = self.dataset.has_sub_dataset(self.sub_dataset)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_subset, self.sub_dataset.res
        self.assertIn(triple, self.dataset.g)

    def test_has_sparql_endpoint(self):
        endpoint = URIRef("http://sparql/")
        result = self.dataset.has_sparql_endpoint(endpoint)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_sparql_endpoint, endpoint
        self.assertIn(triple, self.dataset.g)

    def test_has_distribution(self):
        result = self.dataset.has_distribution(self.di)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_distribution, self.di.res
        self.assertIn(triple, self.dataset.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet
from rdflib import RDF, XSD, Literal


class TestDiscourseElement(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.rp = self.graph_set.add_rp(self.resp_agent)
        self.pl = self.graph_set.add_pl(self.resp_agent)
        self.de1 = self.graph_set.add_de(self.resp_agent)
        self.de2 = self.graph_set.add_de(self.resp_agent)

    def test_has_title(self):
        title = "DiscourseElement"
        result = self.de1.has_title(title)
        self.assertIsNone(result)

        triple = self.de1.res, GraphEntity.iri_title, Literal(title, datatype=XSD.string)
        self.assertIn(triple, self.de1.g)

    def test_contains_discourse_element(self):
        result = self.de1.contains_discourse_element(self.de2)
        self.assertIsNone(result)

        triple = self.de1.res, GraphEntity.iri_contains_de, self.de2.res
        self.assertIn(triple, self.de1.g)

    def test_has_next_de(self):
        result = self.de1.has_next_de(self.de2)
        self.assertIsNone(result)

        triple = self.de1.res, GraphEntity.iri_has_next, self.de2.res
        self.assertIn(triple, self.de1.g)

    def test_is_context_of_rp(self):
        result = self.de1.is_context_of_rp(self.rp)
        self.assertIsNone(result)

        triple = self.de1.res, GraphEntity.iri_is_context_of, self.rp.res
        self.assertIn(triple, self.de1.g)

    def test_is_context_of_pl(self):
        result = self.de1.is_context_of_pl(self.pl)
        self.assertIsNone(result)

        triple = self.de1.res, GraphEntity.iri_is_context_of, self.pl.res
        self.assertIn(triple, self.de1.g)

    def test_has_content(self):
        content = "Content"
        result = self.de1.has_content(content)
        self.assertIsNone(result)

        triple = self.de1.res, GraphEntity.iri_has_content, Literal(content, datatype=XSD.string)
        self.assertIn(triple, self.de1.g)

    def test_create_section(self):
        result = self.de1.create_section()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_section
        self.assertIn(triple, self.de1.g)

    def test_create_section_title(self):
        result = self.de1.create_section_title()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_section_title
        self.assertIn(triple, self.de1.g)

    def test_create_paragraph(self):
        result = self.de1.create_paragraph()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_paragraph
        self.assertIn(triple, self.de1.g)

    def test_create_sentence(self):
        result = self.de1.create_sentence()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_sentence
        self.assertIn(triple, self.de1.g)

    def test_create_text_chunk(self):
        result = self.de1.create_text_chunk()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_text_chunk
        self.assertIn(triple, self.de1.g)

    def test_create_table(self):
        result = self.de1.create_table()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_table
        self.assertIn(triple, self.de1.g)

    def test_create_footnote(self):
        result = self.de1.create_footnote()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_footnote
        self.assertIn(triple, self.de1.g)

    def test_create_caption(self):
        result = self.de1.create_caption()
        self.assertIsNone(result)

        triple = self.de1.res, RDF.type, GraphEntity.iri_caption
        self.assertIn(triple, self.de1.g)

    def test_assign_more_structural_types_de(self):
        result1 = self.de1.create_table()
        self.assertIsNone(result1)
        result2 = self.de1.create_footnote()
        self.assertIsNone(result2)

        triple1 = self.de1.res, RDF.type, GraphEntity.iri_table
        triple2 = self.de1.res, RDF.type, GraphEntity.iri_footnote
        self.assertIn(triple2, self.de1.g) and not self.assertIn(triple1, self.de1.g)

    def test_assign_more_rhetorical_types_de(self):
        result1 = self.de1.create_materials()
        self.assertIsNone(result1)
        result2 = self.de1.create_methods()
        self.assertIsNone(result2)

        triple1 = self.de1.res, RDF.type, GraphEntity.iri_materials
        triple2 = self.de1.res, RDF.type, GraphEntity.iri_methods
        self.assertIn(triple2, self.de1.g) and self.assertIn(triple1, self.de1.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from oc_ocdm.decorators import accepts_only

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef
    from oc_ocdm.graph.entities.bibliographic.citation import Citation
from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.entities.bibliographic_entity import BibliographicEntity


class ReferenceAnnotation(BibliographicEntity):
    """Reference annotation (short: an): an annotation, attached either to an in-text
       reference pointer or to a bibliographic reference, describing the related citation. If an
       in-text reference pointer is annotated, the related citation may be characterized with a
       citation function (the reason for that citation) specific to the textual location of that
       in-text reference pointer within the citing entity. If a bibliographic reference is
       annotated, the related citation may be similarly characterized in a more general way
       with a citation function (the reason for that citation)."""

    @accepts_only('an')
    def merge(self, other: ReferenceAnnotation) -> None:
        """
        The merge operation allows combining two ``ReferenceAnnotation`` entities into a single one,
        by marking the second entity as to be deleted while also copying its data into the current
        ``ReferenceAnnotation``. Moreover, every triple from the containing ``GraphSet`` referring to the second
        entity gets "redirected" to the current entity: **every other reference contained inside a
        different source (e.g. a triplestore) must be manually handled by the user!**

        In case of functional properties, values from the current entity get overwritten
        by those coming from the second entity while, in all other cases, values from the
        second entity are simply appended to those of the current entity. In this context,
        ``rdfs:label`` is considered as a functional property, while ``rdf:type`` is not.

        :param other: The entity which will be marked as to be deleted and whose properties will
         be merged into the current entity.
        :type other: ReferenceAnnotation
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        super(ReferenceAnnotation, self).merge(other)

        citation: Optional[Citation] = other.get_body_annotation()
        if citation is not None:
            self.has_body_annotation(citation)

    # HAS CITATION (Citation)
    def get_body_annotation(self) -> Optional[Citation]:
        """
        Getter method corresponding to the ``oa:hasBody`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_body, 'ci')
        if uri is not None:
            return self.g_set.add_ci(self.resp_agent, self.source, uri)

    @accepts_only('ci')
    def has_body_annotation(self, ci_res: Citation) -> None:
        """
        Setter method corresponding to the ``oa:hasBody`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The citation to which the annotation relates, that is relevant either to a bibliographic
        reference or to an in-text reference pointer that denotes such a bibliographic reference.`

        :param ci_res: The value that will be set as the object of the property related to this method
        :type ci_res: Citation
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_body_annotation()
        self.g.add((self.res, GraphEntity.iri_has_body, ci_res.res))

    def remove_body_annotation(self) -> None:
        """
        Remover method corresponding to the ``oa:hasBody`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_has_body, None))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2023, Arcangelo Massari <arcangelo.massari@unibo.it>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.

import os
import unittest
from unittest.mock import patch

from oc_ocdm.graph import GraphSet
from oc_ocdm.reader import Reader
from rdflib import Graph, URIRef
from SPARQLWrapper import POST, SPARQLWrapper


class TestReader(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.endpoint = 'http://127.0.0.1:8804/sparql'
        cls.resp_agent = 'https://orcid.org/0000-0002-8420-0696'
        BASE = os.path.join('oc_ocdm', 'test', 'reader')
        
        cls.br_file = os.path.abspath(os.path.join(BASE, 'br.nt'))
        cls.ra_file = os.path.abspath(os.path.join(BASE, 'ra.nt'))
        cls.id_file = os.path.abspath(os.path.join(BASE, 'id.nt'))
        
        for file_path in [cls.br_file, cls.ra_file, cls.id_file]:
            if os.path.exists(file_path):
                g = Graph()
                g.parse(file_path, format='nt')
                
                insert_query = "INSERT DATA { GRAPH <https://w3id.org/oc/meta/> {\n"
                for s, p, o in g:
                    insert_query += f"{s.n3()} {p.n3()} {o.n3()} .\n"
                insert_query += "} }"
                
                server = SPARQLWrapper(cls.endpoint)
                server.setMethod(POST)
                server.setQuery(insert_query)
                server.query()
    
    def setUp(self):
        self.reader = Reader()
        self.g_set = GraphSet('https://w3id.org/oc/meta')
    
    def test_import_entity_from_triplestore(self):
        """Test importing a single entity from triplestore."""
        self.reader.import_entity_from_triplestore(
            self.g_set, 
            self.endpoint, 
            URIRef('https://w3id.org/oc/meta/br/0605'), 
            self.resp_agent, 
            False
        )
        self.assertEqual(
            set(str(s) for s in self.g_set.res_to_entity.keys()),
            {'https://w3id.org/oc/meta/br/0605'}
        )
    
    def test_import_entities_from_triplestore_batch(self):
        """Test importing multiple entities in batch."""
        entities = [
            URIRef('https://w3id.org/oc/meta/br/0605'),
            URIRef('https://w3id.org/oc/meta/br/0636066666')
        ]
        
        imported = self.reader.import_entities_from_triplestore(
            self.g_set,
            self.endpoint,
            entities,
            self.resp_agent,
            batch_size=1  # Test with small batch size
        )
        
        # Check if all entities were imported
        imported_uris = set(str(s) for s in self.g_set.res_to_entity.keys())
        expected_uris = set(str(e) for e in entities)
        self.assertEqual(imported_uris, expected_uris)
        
        # Check if number of imported entities matches
        self.assertEqual(len(imported), len(entities))
        
        # Check if specific properties were imported correctly
        br_0605 = self.g_set.get_entity(URIRef('https://w3id.org/oc/meta/br/0605'))
        self.assertIsNotNone(br_0605)
        # Check title
        title = next(br_0605.g.objects(br_0605.res, URIRef('http://purl.org/dc/terms/title')))
        self.assertEqual(str(title), "A Review Of Hemolytic Uremic Syndrome In Patients Treated With Gemcitabine Therapy")
    
    def test_import_invalid_entity(self):
        """Test importing a non-existent entity."""
        with self.assertRaises(ValueError):
            self.reader.import_entity_from_triplestore(
                self.g_set,
                self.endpoint,
                URIRef('https://w3id.org/oc/meta/br/99999'),
                self.resp_agent,
                False
            )
        
    def test_batch_import_with_retry(self):
        """Test batch import with connection failure and retry."""
        entities = [
            URIRef('https://w3id.org/oc/meta/br/0605'),
            URIRef('https://w3id.org/oc/meta/br/0636066666')
        ]
        
        # Mock SPARQLWrapper to fail once then succeed
        with patch('SPARQLWrapper.SPARQLWrapper.queryAndConvert') as mock_query:
            mock_query.side_effect = [
                Exception("Connection failed"),  # First attempt fails
                {'results': {'bindings': []}}    # Second attempt succeeds
            ]
            
            # This should retry and eventually succeed
            with self.assertRaises(ValueError):  # Empty results raise ValueError
                self.reader.import_entities_from_triplestore(
                    self.g_set,
                    self.endpoint,
                    entities,
                    self.resp_agent
                )
            
            # Verify that retry happened
            self.assertEqual(mock_query.call_count, 2)
    
    def test_import_mixed_entity_types(self):
        """Test importing different types of entities in the same batch."""
        entities = [
            URIRef('https://w3id.org/oc/meta/br/0605'),       # Bibliographic Resource
            URIRef('https://w3id.org/oc/meta/id/0605'),       # Identifier
            URIRef('https://w3id.org/oc/meta/re/0605')        # Resource Embodiment
        ]
        
        imported = self.reader.import_entities_from_triplestore(
            self.g_set,
            self.endpoint,
            entities,
            self.resp_agent
        )
        
        # Check if entities of different types were imported correctly
        entity_types = set()
        for entity in imported:
            for _, _, o in entity.g.triples((entity.res, URIRef('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'), None)):
                entity_types.add(str(o))
                
        self.assertTrue(len(entity_types) >= 2)  # Should have multiple types
    
    def test_empty_batch_import(self):
        """Test importing an empty list of entities."""
        with self.assertRaises(ValueError):
            self.reader.import_entities_from_triplestore(
                self.g_set,
                self.endpoint,
                [],
                self.resp_agent
            )

    @classmethod
    def tearDownClass(cls):
        """Clean up the triplestore after tests."""
        delete_query = "CLEAR GRAPH <https://w3id.org/oc/meta/>"
        server = SPARQLWrapper(cls.endpoint)
        server.setMethod(POST)
        server.setQuery(delete_query)
        server.query()


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from rdflib import URIRef, Literal, XSD

from oc_ocdm.metadata.metadata_set import MetadataSet
from oc_ocdm.metadata.metadata_entity import MetadataEntity


class TestBibliographicResource(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.metadata_set = MetadataSet("http://test/", "./info_dir/", False)

    def setUp(self):
        self.di = self.metadata_set.add_di("ocdmTest", self.resp_agent)

    def test_has_title(self):
        title = "Resource"
        result = self.di.has_title(title)
        self.assertIsNone(result)

        triple = self.di.res, MetadataEntity.iri_title, Literal(title, datatype=XSD.string)
        self.assertIn(triple, self.di.g)

    def test_has_description(self):
        description = "Resource"
        result = self.di.has_description(description)
        self.assertIsNone(result)

        triple = self.di.res, MetadataEntity.iri_description, Literal(description, datatype=XSD.string)
        self.assertIn(triple, self.di.g)

    def test_has_publication_date(self):
        string = "2020-05-25T12:12:00"
        result = self.di.has_publication_date(string)
        self.assertIsNone(result)

        triple = self.di.res, MetadataEntity.iri_issued, Literal(string, datatype=XSD.dateTime,
                                                                 normalize=False)
        self.assertIn(triple, self.di.g)

    def test_has_license(self):
        license = URIRef("http://license/")
        result = self.di.has_license(license)
        self.assertIsNone(result)

        triple = self.di.res, MetadataEntity.iri_license, license
        self.assertIn(triple, self.di.g)

    def test_has_download_url(self):
        download_url = URIRef("http://download.here/")
        result = self.di.has_download_url(download_url)
        self.assertIsNone(result)

        triple = self.di.res, MetadataEntity.iri_download_url, download_url
        self.assertIn(triple, self.di.g)

    def test_has_media_type(self):
        media_type = URIRef("http://media.type/")
        result = self.di.has_media_type(media_type)
        self.assertIsNone(result)

        triple = self.di.res, MetadataEntity.iri_media_type, media_type
        self.assertIn(triple, self.di.g)

    def test_has_byte_size(self):
        byte_size = "1024"
        result = self.di.has_byte_size(byte_size)
        self.assertIsNone(result)

        triple = self.di.res, MetadataEntity.iri_byte_size, Literal(byte_size, datatype=XSD.decimal,
                                                                    normalize=False)
        self.assertIn(triple, self.di.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet
from rdflib import XSD, Literal


class TestIdentifier(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.identifier = self.graph_set.add_id(self.resp_agent)

    def test_create_orcid(self):
        orcid = "abcdefghi"
        result = self.identifier.create_orcid(orcid)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(orcid, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_orcid
        self.assertIn(triple, self.identifier.g)

    def test_create_doi(self):
        doi = "abcdefghi"
        result = self.identifier.create_doi(doi)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(doi, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_doi
        self.assertIn(triple, self.identifier.g)

    def test_create_pmid(self):
        pmid = "abcdefghi"
        result = self.identifier.create_pmid(pmid)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(pmid, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_pmid
        self.assertIn(triple, self.identifier.g)

    def test_create_pmcid(self):
        pmcid = "abcdefghi"
        result = self.identifier.create_pmcid(pmcid)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(pmcid, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_pmcid
        self.assertIn(triple, self.identifier.g)

    def test_create_issn(self):
        issn = "abcdefghi"
        result = self.identifier.create_issn(issn)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(issn, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_issn
        self.assertIn(triple, self.identifier.g)

    def test_create_isbn(self):
        isbn = "abcdefghi"
        result = self.identifier.create_isbn(isbn)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(isbn, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_isbn
        self.assertIn(triple, self.identifier.g)

    def test_create_url(self):
        url = "abcdefghi"
        result = self.identifier.create_url(url)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(url, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_url
        self.assertIn(triple, self.identifier.g)

    def test_create_xpath(self):
        xpath = "abcdefghi"
        result = self.identifier.create_xpath(xpath)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(xpath, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_xpath
        self.assertIn(triple, self.identifier.g)

    def test_create_intrepid(self):
        intrepid = "abcdefghi"
        result = self.identifier.create_intrepid(intrepid)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(intrepid, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_intrepid
        self.assertIn(triple, self.identifier.g)

    def test_create_xmlid(self):
        xmlid = "abcdefghi"
        result = self.identifier.create_xmlid(xmlid)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(xmlid, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_xmlid
        self.assertIn(triple, self.identifier.g)

    def test_create_wikidata(self):
        wikidata = "abcdefghi"
        result = self.identifier.create_wikidata(wikidata)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(wikidata, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_wikidata
        self.assertIn(triple, self.identifier.g)

    def test_create_wikipedia(self):
        wikipedia = "abcdefghi"
        result = self.identifier.create_wikipedia(wikipedia)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(wikipedia, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_wikipedia
        self.assertIn(triple, self.identifier.g)

    def test_create_crossref(self):
        crossref = "abcdefghi"
        result = self.identifier.create_crossref(crossref)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(crossref, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_crossref
        self.assertIn(triple, self.identifier.g)

    def test_create_datacite(self):
        datacite = "332"
        result = self.identifier.create_datacite(datacite)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(datacite, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_datacite
        self.assertIn(triple, self.identifier.g)

    def test_create_viaf(self):
        viaf = "abcdefghi"
        result = self.identifier.create_viaf(viaf)
        self.assertIsNone(result)

        triple = self.identifier.res, GraphEntity.iri_has_literal_value, Literal(viaf, datatype=XSD.string)
        self.assertIn(triple, self.identifier.g)

        triple = self.identifier.res, GraphEntity.iri_uses_identifier_scheme, GraphEntity.iri_viaf
        self.assertIn(triple, self.identifier.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import List, Optional


class Reporter(object):
    """This class is used as a metaphoric agent being a reporter"""

    def __init__(self, print_sentences: bool = True, prefix: str = "") -> None:
        self.articles: List[List[str]] = []
        self.last_article: Optional[List[str]] = None
        self.last_sentence: Optional[str] = None
        self.print_sentences: bool = print_sentences
        self.prefix: str = prefix

    def new_article(self) -> None:
        if self.last_article is None or len(self.last_article) > 0:
            self.last_article = []
            self.last_sentence = None
            self.articles.append(self.last_article)
            if self.print_sentences and len(self.last_article) > 0:
                print("\n")

    def add_sentence(self, sentence: str, print_this_sentence: bool = True) -> None:
        cur_sentence: str = self.prefix + sentence
        self.last_sentence = cur_sentence
        self.last_article.append(cur_sentence)
        if self.print_sentences and print_this_sentence:
            print(cur_sentence)

    def get_last_sentence(self) -> Optional[str]:
        return self.last_sentence

    def get_articles_as_string(self) -> str:
        result: str = ""
        for article in self.articles:
            for sentence in article:
                result += sentence + "\n"
            result += "\n"
        return result

    def write_file(self, file_path) -> None:
        with open(file_path, 'wt', encoding='utf-8') as f:
            f.write(self.get_articles_as_string())

    def is_empty(self) -> bool:
        return self.last_sentence is None



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

import os
import re
from datetime import datetime
from typing import TYPE_CHECKING
from rdflib import URIRef, Graph

if TYPE_CHECKING:
    from typing import Optional, List, Tuple, Match, Dict, Set
    from oc_ocdm.graph.entities.bibliographic.bibliographic_resource import BibliographicResource
    from oc_ocdm.graph.entities.bibliographic.responsible_agent import ResponsibleAgent
    from oc_ocdm.graph.entities.bibliographic.agent_role import AgentRole

from urllib.parse import quote

from rdflib import RDF, XSD, Literal


def create_date(date_list: List[Optional[int]] = None) -> Optional[str]:
    string: Optional[str] = None
    if date_list is not None:
        l_date_list: int = len(date_list)
        if l_date_list != 0 and date_list[0] is not None:
            if l_date_list == 3 and \
                    ((date_list[1] is not None and date_list[1] != 1) or
                     (date_list[2] is not None and date_list[2] != 1)):
                string = datetime(date_list[0], date_list[1], date_list[2]).strftime('%Y-%m-%d')
            elif l_date_list == 2 and date_list[1] is not None:
                string = datetime(date_list[0], date_list[1], 1).strftime('%Y-%m')
            else:
                string = datetime(date_list[0], 1, 1).strftime('%Y')
    return string


def get_datatype_from_iso_8601(string: str) -> Tuple[URIRef, str]:
    # Keep only the "yyyy-mm-dd" part of the string
    string = string[:10]

    try:
        date_parts: List[int] = [int(s) for s in string.split(sep='-', maxsplit=2)]
    except ValueError:
        raise ValueError("The provided date string is not ISO-8601 compliant!")

    num_of_parts: int = len(date_parts)
    if num_of_parts == 3:
        return XSD.date, datetime(*date_parts).strftime('%Y-%m-%d')
    elif num_of_parts == 2:
        return XSD.gYearMonth, datetime(*date_parts, 1).strftime('%Y-%m')
    else:
        return XSD.gYear, datetime(*date_parts, 1, 1).strftime('%Y')

def get_ordered_contributors_from_br(br: BibliographicResource,
                                     contributor_type: URIRef):

    ar_list: List[AgentRole] = br.get_contributors()

    list_id: int = 0
    heads: Dict[URIRef, Dict] = {}
    tails: Dict[URIRef, Dict] = {}
    sub_lists: List[Dict] = []
    from_id_to_res_in_heads: Dict[int, URIRef] = {}
    for ar in ar_list:
        role_type: URIRef = ar.get_role_type()
        ra: ResponsibleAgent = ar.get_is_held_by()
        next_ar: AgentRole = ar.get_next()
        if next_ar is not None:
            next_ar_res: Optional[URIRef] = next_ar.res
        else:
            next_ar_res: Optional[URIRef] = None

        if role_type is not None and role_type == contributor_type and ra is not None:
            if next_ar_res is not None and next_ar_res in heads:
                sub_list: Dict = heads[next_ar_res]
                sub_list['list'].insert(0, ra)
                del heads[next_ar_res]
                heads[ar.res] = sub_list
                from_id_to_res_in_heads[sub_list['id']] = ar.res
            elif ar.res is not None and ar.res in tails:
                sub_list: Dict = tails[ar.res]
                sub_list['list'].append(ra)
                del tails[ar.res]

                if next_ar_res is not None:
                    tails[next_ar_res] = sub_list
            else:
                # This AR cannot be inserted into any list, so
                # we need to create an entirely new list for it:
                sub_list: Dict = {'id': list_id, 'list': [ra]}
                list_id += 1
                sub_lists.append(sub_list)

                heads[ar.res] = sub_list
                from_id_to_res_in_heads[sub_list['id']] = ar.res
                if next_ar_res is not None:
                    tails[next_ar_res] = sub_list

    ids_in_heads: Set[int] = {val['id'] for val in heads.values()}
    ids_in_tails: Set[int] = {val['id'] for val in tails.values()}
    diff_set: Set[int] = ids_in_heads - ids_in_tails
    if len(diff_set) == 0:
        # No contributor was found!
        return []
    elif len(diff_set) != 1:
        raise ValueError('A malformed list of AgentRole entities was given.')
    else:
        result_list: List[ResponsibleAgent] = []
        cur_id: int = diff_set.pop()
        already_merged_list_ids: Set[int] = set()
        finished: bool = False
        while not finished:
            found: bool = False
            if cur_id in from_id_to_res_in_heads:
                res: URIRef = from_id_to_res_in_heads[cur_id]
                subl: Dict = heads[res]
                subl_id: int = subl['id']
                if subl_id not in already_merged_list_ids:
                    found = True
                    already_merged_list_ids.add(subl_id)
                    result_list = subl['list'] + result_list

                    # Now we need to get the next cur_id value:
                    if res in tails:
                        cur_id = tails[res]['id']
                    else:
                        finished = True

            if not found:
                raise ValueError('A malformed list of AgentRole entities was given.')

        unmerged_list_ids: Set[int] = ids_in_heads - already_merged_list_ids
        if len(unmerged_list_ids) != 0:
            raise ValueError('A malformed list of AgentRole entities was given.')

        return result_list


def encode_url(u: str) -> str:
    return quote(u, "://")


def create_literal(g: Graph, res: URIRef, p: URIRef, s: str, dt: URIRef = None, nor: bool = True) -> None:
    if not is_string_empty(s):
        dt = dt if dt is not None else XSD.string
        g.add((res, p, Literal(s, datatype=dt, normalize=nor)))


def create_type(g: Graph, res: URIRef, res_type: URIRef) -> None:
    g.add((res, RDF.type, res_type))


def is_string_empty(string: str) -> bool:
    return string is None or string.strip() == ""


# Variable used in several functions
entity_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?((?:[1-9][0-9]*)|(?:\d+-\d+))$"
prov_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?((?:[1-9][0-9]*)|(?:\d+-\d+))/prov/([a-z][a-z])/([1-9][0-9]*)$"


def _get_match(regex: str, group: int, string: str) -> str:
    match: Match = re.match(regex, string)
    if match is not None:
        return match.group(group)
    else:
        return ""


def get_base_iri(res: URIRef) -> str:
    string_iri: str = str(res)
    if "/prov/" in string_iri:
        return _get_match(prov_regex, 1, string_iri)
    else:
        return _get_match(entity_regex, 1, string_iri)


def get_short_name(res: URIRef) -> str:
    string_iri: str = str(res)
    if "/prov/" in string_iri:
        return _get_match(prov_regex, 5, string_iri)
    else:
        return _get_match(entity_regex, 2, string_iri)


def get_prov_subject_short_name(prov_res: URIRef) -> str:
    string_iri: str = str(prov_res)
    if "/prov/" in string_iri:
        return _get_match(prov_regex, 2, string_iri)
    else:
        return ""  # non-provenance entities do not have a prov_subject!


def get_prefix(res: URIRef) -> str:
    string_iri: str = str(res)
    if "/prov/" in string_iri:
        return ""  # provenance entities cannot have a supplier prefix
    else:
        return _get_match(entity_regex, 3, string_iri)


def get_prov_subject_prefix(prov_res: URIRef) -> str:
    string_iri: str = str(prov_res)
    if "/prov/" in string_iri:
        return _get_match(prov_regex, 3, string_iri)
    else:
        return ""  # non-provenance entities do not have a prov_subject!


def get_count(res: URIRef) -> str:
    string_iri: str = str(res)
    if "/prov/" in string_iri:
        return _get_match(prov_regex, 6, string_iri)
    else:
        return _get_match(entity_regex, 4, string_iri)


def get_prov_subject_count(prov_res: URIRef) -> str:
    string_iri: str = str(prov_res)
    if "/prov/" in string_iri:
        return _get_match(prov_regex, 4, string_iri)
    else:
        return ""  # non-provenance entities do not have a prov_subject!


def get_resource_number(res: URIRef) -> int:
    string_iri: str = str(res)
    if "/prov/" in string_iri:
        return int(_get_match(prov_regex, 4, string_iri))
    else:
        return int(_get_match(entity_regex, 4, string_iri))


def find_local_line_id(res: URIRef, n_file_item: int = 1) -> int:
    cur_number: int = get_resource_number(res)

    cur_file_split: int = 0
    while True:
        if cur_number > cur_file_split:
            cur_file_split += n_file_item
        else:
            cur_file_split -= n_file_item
            break

    return cur_number - cur_file_split


def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_split: int,
               n_file_item: int, is_json: bool = True, process_id: int|str = None) -> Tuple[str, str]:
    """
    This function is responsible for looking for the correct JSON file that contains the data related to the
    resource identified by the variable 'string_iri'. This search takes into account the organisation in
    directories and files, as well as the particular supplier prefix for bibliographic entities, if specified.
    In case no supplier prefix is specified, the 'default_dir' (usually set to "_") is used instead.
    """
    string_iri: str = str(res)
    process_id_str: str = f"_{process_id}" if process_id else ""

    if is_dataset(res):
        cur_dir_path: str = (base_dir + re.sub(r"^%s(.*)$" % base_iri, r"\1", string_iri))[:-1]
        # In case of dataset, the file path is different from regular files, e.g.
        # /corpus/br/index.json
        cur_file_path: str = cur_dir_path + os.sep + "index" + process_id_str + ".json"
    else:
        cur_number: int = get_resource_number(res)

        # Find the correct file number where to save the resources
        cur_file_split: int = 0
        while True:
            if cur_number > cur_file_split:
                cur_file_split += n_file_item
            else:
                break

        # The data have been split in multiple directories and it is not something related
        # with the provenance data of the whole corpus (e.g. provenance agents)
        if dir_split and not string_iri.startswith(base_iri + "prov/"):
            # Find the correct directory number where to save the file
            cur_split: int = 0
            while True:
                if cur_number > cur_split:
                    cur_split += dir_split
                else:
                    break

            if "/prov/" in string_iri:  # provenance file of a bibliographic entity
                subj_short_name: str = get_prov_subject_short_name(res)
                short_name: str = get_short_name(res)
                sub_folder: str = get_prov_subject_prefix(res)
                file_extension: str = '.json' if is_json else '.nq'
                if sub_folder == "":
                    sub_folder = default_dir
                if sub_folder == "":
                    sub_folder = "_"  # enforce default value

                cur_dir_path: str = base_dir + subj_short_name + os.sep + sub_folder + \
                    os.sep + str(cur_split) + os.sep + str(cur_file_split) + os.sep + "prov"
                cur_file_path: str = cur_dir_path + os.sep + short_name + process_id_str + file_extension
            else:  # regular bibliographic entity
                short_name: str = get_short_name(res)
                sub_folder: str = get_prefix(res)
                file_extension: str = '.json' if is_json else '.nt'
                if sub_folder == "":
                    sub_folder = default_dir
                if sub_folder == "":
                    sub_folder = "_"  # enforce default value

                cur_dir_path: str = base_dir + short_name + os.sep + sub_folder + os.sep + str(cur_split)
                cur_file_path: str = cur_dir_path + os.sep + str(cur_file_split) + process_id_str + file_extension
        # Enter here if no split is needed
        elif dir_split == 0:
            if "/prov/" in string_iri:
                subj_short_name: str = get_prov_subject_short_name(res)
                short_name: str = get_short_name(res)
                sub_folder: str = get_prov_subject_prefix(res)
                file_extension: str = '.json' if is_json else '.nq'
                if sub_folder == "":
                    sub_folder = default_dir
                if sub_folder == "":
                    sub_folder = "_"  # enforce default value

                cur_dir_path: str = base_dir + subj_short_name + os.sep + sub_folder + \
                    os.sep + str(cur_file_split) + os.sep + "prov"
                cur_file_path: str = cur_dir_path + os.sep + short_name + process_id_str + file_extension
            else:
                short_name: str = get_short_name(res)
                sub_folder: str = get_prefix(res)
                file_extension: str = '.json' if is_json else '.nt'
                if sub_folder == "":
                    sub_folder = default_dir
                if sub_folder == "":
                    sub_folder = "_"  # enforce default value

                cur_dir_path: str = base_dir + short_name + os.sep + sub_folder
                cur_file_path: str = cur_dir_path + os.sep + str(cur_file_split) + process_id_str + file_extension
        # Enter here if the data is about a provenance agent, e.g. /corpus/prov/
        else:
            short_name: str = get_short_name(res)
            prefix: str = get_prefix(res)
            count: str = get_count(res)
            file_extension: str = '.json' if is_json else '.nq'

            cur_dir_path: str = base_dir + short_name
            cur_file_path: str = cur_dir_path + os.sep + prefix + count + process_id_str + file_extension

    return cur_dir_path, cur_file_path

def has_supplier_prefix(res: URIRef, base_iri: str) -> bool:
    string_iri: str = str(res)
    return re.search(r"^%s[a-z][a-z]/0" % base_iri, string_iri) is not None

def build_graph_from_results(results: List[Dict]) -> Graph:
    graph = Graph()
    for triple in results:
        s = URIRef(triple['s']['value'])
        p = URIRef(triple['p']['value'])
        if triple['o']['type'] == 'uri':
            o = URIRef(triple['o']['value'])
        else:
            datatype = triple['o'].get('datatype', None)
            datatype = URIRef(datatype) if datatype is not None else None
            o = Literal(triple['o']['value'], datatype=datatype)
        graph.add((s, p, o))
    return graph


def is_dataset(res: URIRef) -> bool:
    string_iri: str = str(res)
    return re.search(r"^.+/[0-9]+(-[0-9]+)?(/[0-9]+)?$", string_iri) is None


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import json
import os
import unittest
from zipfile import ZipFile
from multiprocessing import Pool
from SPARQLWrapper import POST, SPARQLWrapper

from rdflib import ConjunctiveGraph, URIRef, compare

from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.prov.prov_set import ProvSet
from oc_ocdm.storer import Storer
from oc_ocdm.reader import Reader

from shutil import rmtree


class TestStorer(unittest.TestCase):
    ts = 'http://127.0.0.1:8804/sparql'

    def reset_server(self, server:str=ts) -> None:
        ts = SPARQLWrapper(server)
        for graph in {'https://w3id.org/oc/meta/br/', 'https://w3id.org/oc/meta/ra/', 'https://w3id.org/oc/meta/re/', 'https://w3id.org/oc/meta/id/', 'https://w3id.org/oc/meta/ar/', 'http://default.graph/'}:
            ts.setQuery(f'CLEAR GRAPH <{graph}>')
            ts.setMethod(POST)
            ts.query()

    def setUp(self):
        self.resp_agent = "http://resp_agent.test/"
        self.base_iri = "http://test/"
        self.ts = self.ts
        self.graph_set = GraphSet(self.base_iri, "", "060", False)
        self.prov_set = ProvSet(self.graph_set, self.base_iri, "", False)
        self.br = self.graph_set.add_br(self.resp_agent)
        self.data_dir = os.path.join("oc_ocdm", "test", "storer", "data")
        self.prov_dir = os.path.join("oc_ocdm", "test", "storer", "test_provenance")
        self.info_dir = os.path.join(self.prov_dir, "info_dir")
        self.reset_server()

    def tearDown(self):
        if os.path.exists(self.data_dir):
            rmtree(self.data_dir)
        if os.path.exists(self.prov_dir):
            rmtree(os.path.join(self.prov_dir))

    def test_store_graphs_in_file(self):
        base_dir = os.path.join("oc_ocdm", "test", "storer", "data", "rdf") + os.sep
        with self.subTest("output_format=json-ld, zip_output=True"):
            modified_entities = self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=True)
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=True, modified_entities=modified_entities)
            storer.store_all(base_dir, self.base_iri)
            prov_storer.store_all(base_dir, self.base_iri)
            self.graph_set.commit_changes()
            with ZipFile(os.path.join(base_dir, "br", "060", "10000", "1000.zip"), mode="r") as archive:
                with archive.open("1000.json") as f:
                    data = json.load(f)
                    self.assertEqual(data, [{'@graph': [{'@id': 'http://test/br/0601', '@type': ['http://purl.org/spar/fabio/Expression']}], '@id': 'http://test/br/'}])
            with ZipFile(os.path.join(base_dir, "br", "060", "10000", "1000", "prov", "se.zip"), mode="r") as archive:
                with archive.open("se.json") as f:
                    data = [{g:[{k:v for k,v in datum.items() if k != "http://www.w3.org/ns/prov#generatedAtTime"} for datum in data] if g == "@graph" else data for g, data in graph.items()} for graph in json.load(f)]
                    self.assertEqual(data, [{'@graph': [{
                        '@id': 'http://test/br/0601/prov/se/1', 
                        '@type': ['http://www.w3.org/ns/prov#Entity'], 
                        'http://purl.org/dc/terms/description': [{'@value': "The entity 'http://test/br/0601' has been created."}], 
                        'http://www.w3.org/ns/prov#specializationOf': [{'@id': 'http://test/br/0601'}], 
                        'http://www.w3.org/ns/prov#wasAttributedTo': [{'@id': 'http://resp_agent.test/'}]}], '@id': 'http://test/br/0601/prov/'}])
        with self.subTest("output_format=json-ld, zip_output=False"):
            base_dir_1 = os.path.join("oc_ocdm", "test", "storer", "data", "rdf_1") + os.sep
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
            self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
            storer.store_all(base_dir_1, self.base_iri)
            prov_storer.store_all(base_dir_1, self.base_iri)
            self.graph_set.commit_changes()
            with open(os.path.join(base_dir_1, "br", "060", "10000", "1000.json")) as f:
                data = json.load(f)
                self.assertEqual(data, [{'@graph': [{'@id': 'http://test/br/0601', '@type': ['http://purl.org/spar/fabio/Expression']}], '@id': 'http://test/br/'}])
            with open(os.path.join(base_dir_1, "br", "060", "10000", "1000", "prov", "se.json")) as f:
                data = [{g:[{k:v for k,v in datum.items() if k != "http://www.w3.org/ns/prov#generatedAtTime"} for datum in data] if g == "@graph" else data for g, data in graph.items()} for graph in json.load(f)]
                self.assertEqual(data, [{'@graph': [{
                    '@id': 'http://test/br/0601/prov/se/1', 
                    '@type': ['http://www.w3.org/ns/prov#Entity'], 
                    'http://purl.org/dc/terms/description': [{'@value': "The entity 'http://test/br/0601' has been created."}], 
                    'http://www.w3.org/ns/prov#specializationOf': [{'@id': 'http://test/br/0601'}], 
                    'http://www.w3.org/ns/prov#wasAttributedTo': [{'@id': 'http://resp_agent.test/'}]}], '@id': 'http://test/br/0601/prov/'}])
        with self.subTest("output_format=nquads, zip_output=True"):
            base_dir_2 = os.path.join("oc_ocdm", "test", "storer", "data", "rdf_2") + os.sep
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=True)
            self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=True)
            storer.store_all(base_dir_2, self.base_iri)
            prov_storer.store_all(base_dir_2, self.base_iri)
            self.graph_set.commit_changes()
            with ZipFile(os.path.join(base_dir_2, "br", "060", "10000", "1000.zip"), mode="r") as archive:
                with archive.open("1000.nt") as f:
                    data = f.read().decode("utf-8")
                    self.assertEqual(data, "<http://test/br/0601> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://purl.org/spar/fabio/Expression> <http://test/br/> .\n\n")
            with ZipFile(os.path.join(base_dir_2, "br", "060", "10000", "1000", "prov", "se.zip"), mode="r") as archive:
                with archive.open("se.nq") as f:
                    data = f.read().decode("utf-8")
                    data_g = ConjunctiveGraph()
                    expected_data_g = ConjunctiveGraph()
                    data_g.parse(data=data, format="nquads")
                    expected_data_g.parse(data="""
                        <http://test/br/0601/prov/se/1> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Entity> <http://test/br/0601/prov/> .
                        <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#specializationOf> <http://test/br/0601> <http://test/br/0601/prov/> .
                        <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#wasAttributedTo> <http://resp_agent.test/> <http://test/br/0601/prov/> .
                        <http://test/br/0601/prov/se/1> <http://purl.org/dc/terms/description> "The entity 'http://test/br/0601' has been created."^^<http://www.w3.org/2001/XMLSchema#string> <http://test/br/0601/prov/> .
                    """, format="nquads")
                    for s, p, o, c in data_g.quads():
                        if p == URIRef("http://www.w3.org/ns/prov#generatedAtTime"):
                            data_g.remove((s, p, o, c))
                    self.assertTrue(compare.isomorphic(data_g, expected_data_g))
        with self.subTest("output_format=nquads, zip_output=False"):
            base_dir_3 = os.path.join("oc_ocdm", "test", "storer", "data", "rdf_3") + os.sep
            storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=False)
            self.prov_set.generate_provenance()
            prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='nquads', zip_output=False)
            storer.store_all(base_dir_3, self.base_iri)
            prov_storer.store_all(base_dir_3, self.base_iri)
            self.graph_set.commit_changes()
            prov_unzipped = ConjunctiveGraph()
            expected_prov_unzipped = ConjunctiveGraph()
            with open(os.path.join(base_dir_3, "br", "060", "10000", "1000.nt"), "r", encoding="utf-8") as f:
                data_unzipped = f.read()
            prov_unzipped.parse(source=os.path.join(base_dir_3, "br", "060", "10000", "1000", "prov", "se.nq"), format="nquads")
            expected_prov_unzipped.parse(data="""
                <http://test/br/0601/prov/se/1> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Entity> <http://test/br/0601/prov/> .
                <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#specializationOf> <http://test/br/0601> <http://test/br/0601/prov/> .
                <http://test/br/0601/prov/se/1> <http://www.w3.org/ns/prov#wasAttributedTo> <http://resp_agent.test/> <http://test/br/0601/prov/> .
                <http://test/br/0601/prov/se/1> <http://purl.org/dc/terms/description> "The entity 'http://test/br/0601' has been created."^^<http://www.w3.org/2001/XMLSchema#string> <http://test/br/0601/prov/> .
            """, format="nquads")
            for s, p, o, c in prov_unzipped.quads():
                if p == URIRef("http://www.w3.org/ns/prov#generatedAtTime"):
                    prov_unzipped.remove((s, p, o, c))
            self.assertEqual(data_unzipped, "<http://test/br/0601> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://purl.org/spar/fabio/Expression> <http://test/br/> .\n\n")
            self.assertTrue(compare.isomorphic(prov_unzipped, expected_prov_unzipped))

    def test_store_graphs_in_file_multiprocessing(self):
        base_dir = os.path.join("oc_ocdm", "test", "storer", "data", "multiprocessing") + os.sep
        storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
        self.prov_set.generate_provenance()
        prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
        storer.store_all(base_dir, self.base_iri, process_id=7)
        prov_storer.store_all(base_dir, self.base_iri, process_id=7)
        with open(os.path.join(base_dir, "br", "060", "10000", "1000_7.json")) as f:
            data = json.load(f)
            self.assertEqual(data, [{'@graph': [{'@id': 'http://test/br/0601', '@type': ['http://purl.org/spar/fabio/Expression']}], '@id': 'http://test/br/'}])
        with open(os.path.join(base_dir, "br", "060", "10000", "1000", "prov", "se_7.json")) as f:
            data = [{g:[{k:v for k,v in datum.items() if k != "http://www.w3.org/ns/prov#generatedAtTime"} for datum in data] if g == "@graph" else data for g, data in graph.items()} for graph in json.load(f)]
            self.assertEqual(data, [{'@graph': [{
                '@id': 'http://test/br/0601/prov/se/1', 
                '@type': ['http://www.w3.org/ns/prov#Entity'], 
                'http://purl.org/dc/terms/description': [{'@value': "The entity 'http://test/br/0601' has been created."}], 
                'http://www.w3.org/ns/prov#specializationOf': [{'@id': 'http://test/br/0601'}], 
                'http://www.w3.org/ns/prov#wasAttributedTo': [{'@id': 'http://resp_agent.test/'}]}], '@id': 'http://test/br/0601/prov/'}])

    def test_provenance(self):
        graph_set = GraphSet(self.base_iri, "", "060", False)
        prov_set = ProvSet(graph_set, self.base_iri, info_dir=self.info_dir)
        base_dir = os.path.join("oc_ocdm", "test", "storer", "test_provenance") + os.sep
        graph_set.add_br(self.resp_agent)
        graph_set.add_br(self.resp_agent)
        graph_set.add_br(self.resp_agent)
        prov_set.generate_provenance()
        storer = Storer(graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
        prov_storer = Storer(prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
        prov_storer.store_all(base_dir, self.base_iri)
        storer.upload_all(self.ts, base_dir)
        graph_set.commit_changes()
        entities_to_process = [('http://test/br/0601',), ('http://test/br/0602',), ('http://test/br/0603',)]
        with Pool(processes=3) as pool:
            pool.starmap(process_entity, entities_to_process)

    def test_store_graphs_save_queries(self):
        base_dir = os.path.join("oc_ocdm", "test", "storer", "data", "rdf_save_queries") + os.sep
        storer = Storer(self.graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
        self.prov_set.generate_provenance()
        prov_storer = Storer(self.prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
        storer.store_all(base_dir, self.base_iri)
        prov_storer.store_all(base_dir, self.base_iri)
        
        to_be_uploaded_dir = os.path.join(base_dir, "to_be_uploaded")
        storer.upload_all(self.ts, base_dir, save_queries=True)
        
        # Controlla che la directory to_be_uploaded esista
        self.assertTrue(os.path.exists(to_be_uploaded_dir))
        
        # Controlla che ci sia almeno un file nella directory to_be_uploaded
        saved_queries = os.listdir(to_be_uploaded_dir)
        self.assertGreater(len(saved_queries), 0)
        
        # Controlla il contenuto di uno dei file salvati
        query_file = os.path.join(to_be_uploaded_dir, saved_queries[0])
        with open(query_file, 'r', encoding='utf-8') as f:
            query_content = f.read()
            self.assertIn("INSERT DATA", query_content)  # Verifica che ci sia una query di inserimento

def process_entity(entity):
    base_iri = "http://test/"
    ts = 'http://127.0.0.1:8804/sparql'
    resp_agent = "http://resp_agent.test/"
    base_dir = os.path.join("oc_ocdm", "test", "storer", "test_provenance") + os.sep
    info_dir = os.path.join("oc_ocdm", "test", "storer", "test_provenance", "info_dir")
    graph_set = GraphSet(base_iri, "", "060", False)
    Reader.import_entity_from_triplestore(graph_set, ts, URIRef(entity), resp_agent)
    br = graph_set.get_entity(URIRef(entity))
    br.has_title("Hola")
    prov_set = ProvSet(graph_set, base_iri, info_dir=info_dir)
    prov_set.generate_provenance()
    storer = Storer(graph_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
    prov_storer = Storer(prov_set, context_map={}, dir_split=10000, n_file_item=1000, default_dir="_", output_format='json-ld', zip_output=False)
    prov_storer.store_all(base_dir, base_iri)
    storer.upload_all(ts, base_dir)


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet
from rdflib import XSD, Literal


class TestReferencePointer(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.an = self.graph_set.add_an(self.resp_agent)
        self.rp1 = self.graph_set.add_rp(self.resp_agent)
        self.rp2 = self.graph_set.add_rp(self.resp_agent)
        self.be = self.graph_set.add_be(self.resp_agent)

    def test_has_content(self):
        content = "Content"
        result = self.rp1.has_content(content)
        self.assertIsNone(result)

        triple = self.rp1.res, GraphEntity.iri_has_content, Literal(content, datatype=XSD.string)
        self.assertIn(triple, self.rp1.g)

    def test_has_next_rp(self):
        result = self.rp1.has_next_rp(self.rp2)
        self.assertIsNone(result)

        triple = self.rp1.res, GraphEntity.iri_has_next, self.rp2.res
        self.assertIn(triple, self.rp1.g)

    def test_denotes_be(self):
        result = self.rp1.denotes_be(self.be)
        self.assertIsNone(result)

        triple = self.rp1.res, GraphEntity.iri_denotes, self.be.res
        self.assertIn(triple, self.rp1.g)

    def test_has_annotation(self):
        result = self.rp1.has_annotation(self.an)
        self.assertIsNone(result)

        triple = self.rp1.res, GraphEntity.iri_has_annotation, self.an.res
        self.assertIn(triple, self.rp1.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from oc_ocdm.counter_handler.in_memory_counter_handler import InMemoryCounterHandler


class TestFilesystemCounterHandler(unittest.TestCase):

    def setUp(self) -> None:
        self.counter_handler = InMemoryCounterHandler()

    def test_set_counter(self):
        with self.subTest("Set BR counter"):
            count = 99
            self.counter_handler.entity_counters["br"] = count

            new_count = 205
            result = self.counter_handler.set_counter(new_count, "br")
            self.assertIsNone(result)
            self.assertEqual(self.counter_handler.entity_counters["br"], new_count)
        with self.subTest("Set BR counter, long number"):
            long_count = 2**256
            self.counter_handler.entity_counters["br"] = long_count

            new_count = 2**512
            result = self.counter_handler.set_counter(new_count, "br")
            self.assertIsNone(result)
            self.assertEqual(self.counter_handler.entity_counters["br"], new_count)
        with self.subTest("Set SE counter"):
            count = 99
            identifier = 1234
            counter_list = [0]*identifier
            counter_list[-1] = count
            self.counter_handler.prov_counters["br"]["se"] = counter_list

            new_count = 205
            result = self.counter_handler.set_counter(new_count, "br", "se", identifier)
            self.assertIsNone(result)
            self.assertEqual(self.counter_handler.prov_counters["br"]["se"][identifier - 1], new_count)
        with self.subTest("Set SE counter, long number"):
            long_count = 2**256
            identifier = 1234
            counter_list = [0]*identifier
            counter_list[-1] = long_count
            self.counter_handler.prov_counters["br"]["se"] = counter_list

            new_count = 2**512
            result = self.counter_handler.set_counter(new_count, "br", "se", identifier)
            self.assertIsNone(result)
            self.assertEqual(self.counter_handler.prov_counters["br"]["se"][identifier - 1], new_count)
        with self.subTest("Wrong inputs"):
            self.assertRaises(ValueError, self.counter_handler.set_counter, -1, "xyz")
            self.assertRaises(ValueError, self.counter_handler.set_counter, 1, "xyz")
            self.assertRaises(ValueError, self.counter_handler.set_counter, 1, "br", "xyz")
            self.assertRaises(ValueError, self.counter_handler.set_counter, 1, "br", "se", -1)

    def test_read_counter(self):
        with self.subTest("Read BR counter"):
            count = 99
            self.counter_handler.entity_counters["br"] = count

            result = self.counter_handler.read_counter("br")
            self.assertIsNotNone(result)
            self.assertEqual(result, count)
        with self.subTest("Read BR counter, long number"):
            long_count = 2**256
            self.counter_handler.entity_counters["br"] = long_count

            result = self.counter_handler.read_counter("br")
            self.assertIsNotNone(result)
            self.assertEqual(result, long_count)
        with self.subTest("Read SE counter"):
            count = 99
            identifier = 1234
            counter_list = [0]*identifier
            counter_list[-1] = count
            self.counter_handler.prov_counters["br"]["se"] = counter_list

            result = self.counter_handler.read_counter("br", "se", identifier)
            self.assertIsNotNone(result)
            self.assertEqual(result, count)
        with self.subTest("Read SE counter, long number"):
            long_count = 2**256
            identifier = 1234
            counter_list = [0]*identifier
            counter_list[-1] = long_count
            self.counter_handler.prov_counters["br"]["se"] = counter_list

            result = self.counter_handler.read_counter("br", "se", identifier)
            self.assertIsNotNone(result)
            self.assertEqual(result, long_count)
        with self.subTest("Wrong inputs"):
            self.assertRaises(ValueError, self.counter_handler.read_counter, "xyz")
            self.assertRaises(ValueError, self.counter_handler.read_counter, "br", "xyz")
            self.assertRaises(ValueError, self.counter_handler.read_counter, "br", "se", -1)

    def test_increment_counter(self):
        with self.subTest("Increment BR counter"):
            count = 99
            self.counter_handler.entity_counters["br"] = count

            result = self.counter_handler.increment_counter("br")
            self.assertIsNotNone(result)
            self.assertEqual(result, count + 1)
            self.assertEqual(self.counter_handler.entity_counters["br"], count + 1)
        with self.subTest("Increment BR counter, long number"):
            long_count = 2**256
            self.counter_handler.entity_counters["br"] = long_count

            result = self.counter_handler.increment_counter("br")
            self.assertIsNotNone(result)
            self.assertEqual(result, long_count + 1)
            self.assertEqual(self.counter_handler.entity_counters["br"], long_count + 1)
        with self.subTest("Increment SE counter"):
            count = 99
            identifier = 1234
            counter_list = [0]*identifier
            counter_list[-1] = count
            self.counter_handler.prov_counters["br"]["se"] = counter_list

            result = self.counter_handler.increment_counter("br", "se", identifier)
            self.assertIsNotNone(result)
            self.assertEqual(result, count + 1)
            self.assertEqual(self.counter_handler.prov_counters["br"]["se"][identifier - 1], count + 1)
        with self.subTest("Increment SE counter, long number"):
            long_count = 2**256
            identifier = 1234
            counter_list = [0]*identifier
            counter_list[-1] = long_count
            self.counter_handler.prov_counters["br"]["se"] = counter_list

            result = self.counter_handler.increment_counter("br", "se", identifier)
            self.assertIsNotNone(result)
            self.assertEqual(result, long_count + 1)
            self.assertEqual(self.counter_handler.prov_counters["br"]["se"][identifier - 1], long_count + 1)
        with self.subTest("Wrong inputs"):
            self.assertRaises(ValueError, self.counter_handler.increment_counter, "xyz")
            self.assertRaises(ValueError, self.counter_handler.increment_counter, "br", "xyz")
            self.assertRaises(ValueError, self.counter_handler.increment_counter, "br", "se", -1)

    def test_set_metadata_counter(self):
        dataset_name: str = "http://dataset/"
        with self.subTest("Set DI counter"):
            count = 99
            self.counter_handler.metadata_counters[dataset_name] = {"di": count}

            new_count = 205
            result = self.counter_handler.set_metadata_counter(new_count, "di", dataset_name)
            self.assertIsNone(result)
            self.assertEqual(self.counter_handler.metadata_counters[dataset_name]["di"], new_count)
        with self.subTest("Set DI counter, long number"):
            long_count = 2 ** 256
            self.counter_handler.metadata_counters[dataset_name] = {"di": long_count}

            new_count = 2**512
            result = self.counter_handler.set_metadata_counter(new_count, "di", dataset_name)
            self.assertIsNone(result)
            self.assertEqual(self.counter_handler.metadata_counters[dataset_name]["di"], new_count)
        with self.subTest("Wrong inputs"):
            self.assertRaises(ValueError, self.counter_handler.set_metadata_counter, -1, "xyz", dataset_name)
            self.assertRaises(ValueError, self.counter_handler.set_metadata_counter, 1, "xyz", dataset_name)
            self.assertRaises(ValueError, self.counter_handler.set_metadata_counter, 1, "di", None)

    def test_read_metadata_counter(self):
        dataset_name: str = "http://dataset/"
        with self.subTest("Read DI counter"):
            count = 99
            self.counter_handler.metadata_counters[dataset_name] = {"di": count}

            result = self.counter_handler.read_metadata_counter("di", dataset_name)
            self.assertIsNotNone(result)
            self.assertEqual(result, count)
        with self.subTest("Read DI counter, long number"):
            long_count = 2**256
            self.counter_handler.metadata_counters[dataset_name] = {"di": long_count}

            result = self.counter_handler.read_metadata_counter("di", dataset_name)
            self.assertIsNotNone(result)
            self.assertEqual(result, long_count)
        with self.subTest("Wrong inputs"):
            self.assertRaises(ValueError, self.counter_handler.read_metadata_counter, "xyz", dataset_name)
            self.assertRaises(ValueError, self.counter_handler.read_metadata_counter, "di", None)

    def test_increment_metadata_counter(self):
        dataset_name: str = "http://dataset/"
        with self.subTest("Increment DI counter"):
            count = 99
            self.counter_handler.metadata_counters[dataset_name] = {"di": count}

            result = self.counter_handler.increment_metadata_counter("di", dataset_name)
            self.assertIsNotNone(result)
            self.assertEqual(result, count + 1)
            self.assertEqual(self.counter_handler.metadata_counters[dataset_name]["di"], count + 1)
        with self.subTest("Increment DI counter, long number"):
            long_count = 2**256
            self.counter_handler.metadata_counters[dataset_name] = {"di": long_count}

            result = self.counter_handler.increment_metadata_counter("di", dataset_name)
            self.assertIsNotNone(result)
            self.assertEqual(result, long_count + 1)
            self.assertEqual(self.counter_handler.metadata_counters[dataset_name]["di"], long_count + 1)
        with self.subTest("Wrong inputs"):
            self.assertRaises(ValueError, self.counter_handler.increment_metadata_counter, "xyz", dataset_name)
            self.assertRaises(ValueError, self.counter_handler.increment_metadata_counter, "di", None)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from rdflib import Graph

from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.graph.entities.identifier import Identifier
from oc_ocdm.graph.entities.bibliographic.agent_role import AgentRole
from oc_ocdm.graph.entities.bibliographic.bibliographic_reference import BibliographicReference
from oc_ocdm.graph.entities.bibliographic.bibliographic_resource import BibliographicResource
from oc_ocdm.graph.entities.bibliographic.citation import Citation
from oc_ocdm.graph.entities.bibliographic.discourse_element import DiscourseElement
from oc_ocdm.graph.entities.bibliographic.pointer_list import PointerList
from oc_ocdm.graph.entities.bibliographic.reference_annotation import ReferenceAnnotation
from oc_ocdm.graph.entities.bibliographic.reference_pointer import ReferencePointer
from oc_ocdm.graph.entities.bibliographic.resource_embodiment import ResourceEmbodiment
from oc_ocdm.graph.entities.bibliographic.responsible_agent import ResponsibleAgent


class TestGraphSet(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    def setUp(self):
        self.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def test_get_entity(self):
        ar = self.graph_set.add_ar(self.resp_agent)
        ref = ar.res
        result = self.graph_set.get_entity(ref)
        self.assertIsNotNone(result)
        self.assertIs(result, ar)

    def test_add_an(self):
        an = self.graph_set.add_an(self.resp_agent)

        self.assertIsNotNone(an)
        self.assertIsInstance(an, ReferenceAnnotation)
        self.assertEqual(str(an.g.identifier), self.graph_set.g_an)

    def test_add_ar(self):
        ar = self.graph_set.add_ar(self.resp_agent)

        self.assertIsNotNone(ar)
        self.assertIsInstance(ar, AgentRole)
        self.assertEqual(str(ar.g.identifier), self.graph_set.g_ar)

    def test_add_be(self):
        be = self.graph_set.add_be(self.resp_agent)

        self.assertIsNotNone(be)
        self.assertIsInstance(be, BibliographicReference)
        self.assertEqual(str(be.g.identifier), self.graph_set.g_be)

    def test_add_br(self):
        br = self.graph_set.add_br(self.resp_agent)

        self.assertIsNotNone(br)
        self.assertIsInstance(br, BibliographicResource)
        self.assertEqual(str(br.g.identifier), self.graph_set.g_br)

    def test_add_ci(self):
        ci = self.graph_set.add_ci(self.resp_agent)

        self.assertIsNotNone(ci)
        self.assertIsInstance(ci, Citation)
        self.assertEqual(str(ci.g.identifier), self.graph_set.g_ci)

    def test_add_de(self):
        de = self.graph_set.add_de(self.resp_agent)

        self.assertIsNotNone(de)
        self.assertIsInstance(de, DiscourseElement)
        self.assertEqual(str(de.g.identifier), self.graph_set.g_de)

    def test_add_id(self):
        identifier = self.graph_set.add_id(self.resp_agent)

        self.assertIsNotNone(identifier)
        self.assertIsInstance(identifier, Identifier)
        self.assertEqual(str(identifier.g.identifier), self.graph_set.g_id)

    def test_add_pl(self):
        pl = self.graph_set.add_pl(self.resp_agent)

        self.assertIsNotNone(pl)
        self.assertIsInstance(pl, PointerList)
        self.assertEqual(str(pl.g.identifier), self.graph_set.g_pl)

    def test_add_rp(self):
        rp = self.graph_set.add_rp(self.resp_agent)

        self.assertIsNotNone(rp)
        self.assertIsInstance(rp, ReferencePointer)
        self.assertEqual(str(rp.g.identifier), self.graph_set.g_rp)

    def test_add_ra(self):
        ra = self.graph_set.add_ra(self.resp_agent)

        self.assertIsNotNone(ra)
        self.assertIsInstance(ra, ResponsibleAgent)
        self.assertEqual(str(ra.g.identifier), self.graph_set.g_ra)

    def test_add_re(self):
        re = self.graph_set.add_re(self.resp_agent)

        self.assertIsNotNone(re)
        self.assertIsInstance(re, ResourceEmbodiment)
        self.assertEqual(str(re.g.identifier), self.graph_set.g_re)

    def test_graphs(self):
        count = 10
        for i in range(count):
            self.graph_set.add_ar(self.resp_agent)
        result = self.graph_set.graphs()
        self.assertIsNotNone(result)
        self.assertEqual(len(result), count)
        for graph in result:
            self.assertIsInstance(graph, Graph)

    def test_get_graph_iri(self):
        ar = self.graph_set.add_ar(self.resp_agent)
        iri = str(ar.g.identifier)
        result = GraphSet.get_graph_iri(ar.g)
        self.assertIsNotNone(result)
        self.assertEqual(iri, result)

    def test_get_orphans(self):
        br = self.graph_set.add_br(self.resp_agent)
        ar = self.graph_set.add_ar(self.resp_agent)
        ra = self.graph_set.add_ra(self.resp_agent)

        with self.subTest("subtest 1"):
            orphans = self.graph_set.get_orphans()
            self.assertIsNotNone(orphans)

            orphans_set = {o.res for o in orphans}
            self.assertSetEqual({br.res, ar.res, ra.res}, orphans_set)

        with self.subTest("subtest 2"):
            # Here we link br and ar:
            br.has_contributor(ar)

            orphans = self.graph_set.get_orphans()
            self.assertIsNotNone(orphans)

            orphans_set = {o.res for o in orphans}
            self.assertSetEqual({br.res, ra.res}, orphans_set)

        with self.subTest("subtest 3"):
            # Here we link ar and ra:
            ar.is_held_by(ra)

            orphans = self.graph_set.get_orphans()
            self.assertIsNotNone(orphans)

            orphans_set = {o.res for o in orphans}
            self.assertSetEqual({br.res}, orphans_set)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from oc_ocdm.abstract_set import AbstractSet
from oc_ocdm.reader import Reader
from oc_ocdm.support.support import get_count, get_prefix, get_short_name
from SPARQLWrapper import RDFXML, SPARQLWrapper

if TYPE_CHECKING:
    from typing import Dict, ClassVar, Tuple, Optional, List, Set
    from rdflib import ConjunctiveGraph

from oc_ocdm.counter_handler.counter_handler import CounterHandler
from oc_ocdm.counter_handler.filesystem_counter_handler import \
    FilesystemCounterHandler
from oc_ocdm.counter_handler.in_memory_counter_handler import \
    InMemoryCounterHandler
from oc_ocdm.graph.entities.bibliographic.agent_role import AgentRole
from oc_ocdm.graph.entities.bibliographic.bibliographic_reference import \
    BibliographicReference
from oc_ocdm.graph.entities.bibliographic.bibliographic_resource import \
    BibliographicResource
from oc_ocdm.graph.entities.bibliographic.citation import Citation
from oc_ocdm.graph.entities.bibliographic.discourse_element import \
    DiscourseElement
from oc_ocdm.graph.entities.bibliographic.pointer_list import PointerList
from oc_ocdm.graph.entities.bibliographic.reference_annotation import \
    ReferenceAnnotation
from oc_ocdm.graph.entities.bibliographic.reference_pointer import \
    ReferencePointer
from oc_ocdm.graph.entities.bibliographic.resource_embodiment import \
    ResourceEmbodiment
from oc_ocdm.graph.entities.bibliographic.responsible_agent import \
    ResponsibleAgent
from oc_ocdm.graph.entities.identifier import Identifier
from oc_ocdm.graph.graph_entity import GraphEntity
from rdflib import Graph, Namespace, URIRef


class GraphSet(AbstractSet):
    # Labels
    labels: ClassVar[Dict[str, str]] = {
        "an": "annotation",
        "ar": "agent role",
        "be": "bibliographic entry",
        "br": "bibliographic resource",
        "ci": "citation",
        "de": "discourse element",
        "id": "identifier",
        "pl": "single location pointer list",
        "ra": "responsible agent",
        "re": "resource embodiment",
        "rp": "in-text reference pointer"
    }

    def __init__(self, base_iri: str, info_dir: str = "", supplier_prefix: str = "",
                 wanted_label: bool = True, custom_counter_handler: CounterHandler = None) -> None:
        super(GraphSet, self).__init__()
        # The following variable maps a URIRef with the related graph entity
        self.res_to_entity: Dict[URIRef, GraphEntity] = {}
        self.base_iri: str = base_iri
        self.info_dir: str = info_dir
        self.supplier_prefix: str = supplier_prefix
        self.wanted_label: bool = wanted_label
        # Graphs
        # The following structure of URL is quite important for the other classes
        # developed and should not be changed. The only part that can change is the
        # value of the base_iri
        self.g_an: str = base_iri + "an/"
        self.g_ar: str = base_iri + "ar/"
        self.g_be: str = base_iri + "be/"
        self.g_br: str = base_iri + "br/"
        self.g_ci: str = base_iri + "ci/"
        self.g_de: str = base_iri + "de/"
        self.g_id: str = base_iri + "id/"
        self.g_pl: str = base_iri + "pl/"
        self.g_ra: str = base_iri + "ra/"
        self.g_re: str = base_iri + "re/"
        self.g_rp: str = base_iri + "rp/"

        if custom_counter_handler:
            self.counter_handler = custom_counter_handler
        elif info_dir is not None and info_dir != "":
            self.counter_handler = FilesystemCounterHandler(info_dir, supplier_prefix)
        else:
            self.counter_handler = InMemoryCounterHandler()

    def get_entity(self, res: URIRef) -> Optional[GraphEntity]:
        if res in self.res_to_entity:
            return self.res_to_entity[res]

    # Add resources related to bibliographic entities
    def add_an(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> ReferenceAnnotation:
        if res is not None and get_short_name(res) != "an":
            raise ValueError(f"Given res: <{res}> is inappropriate for a ReferenceAnnotation entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_an, "an", res)
        return ReferenceAnnotation(cur_g, self, res, GraphEntity.iri_note,
                                   resp_agent, source, count, label, "an",
                                   preexisting_graph)

    def add_ar(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> AgentRole:
        if res is not None and get_short_name(res) != "ar":
            raise ValueError(f"Given res: <{res}> is inappropriate for an AgentRole entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_ar, "ar", res)
        return AgentRole(cur_g, self, res, GraphEntity.iri_role_in_time,
                         resp_agent, source, count, label, "ar",
                         preexisting_graph)

    def add_be(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> BibliographicReference:
        if res is not None and get_short_name(res) != "be":
            raise ValueError(f"Given res: <{res}> is inappropriate for a BibliographicReference entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_be, "be", res)
        return BibliographicReference(cur_g, self, res, GraphEntity.iri_bibliographic_reference,
                                      resp_agent, source, count, label, "be",
                                      preexisting_graph)

    def add_br(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> BibliographicResource:
        if res is not None and get_short_name(res) != "br":
            raise ValueError(f"Given res: <{res}> is inappropriate for a BibliographicResource entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_br, "br", res)
        return BibliographicResource(cur_g, self, res, GraphEntity.iri_expression,
                                     resp_agent, source, count, label, "br",
                                     preexisting_graph)

    def add_ci(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> Citation:
        if res is not None and get_short_name(res) != "ci":
            raise ValueError(f"Given res: <{res}> is inappropriate for a Citation entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_ci, "ci", res)
        return Citation(cur_g, self, res, GraphEntity.iri_citation,
                        resp_agent, source, count, label, "ci",
                        preexisting_graph)

    def add_de(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> DiscourseElement:
        if res is not None and get_short_name(res) != "de":
            raise ValueError(f"Given res: <{res}> is inappropriate for a DiscourseElement entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_de, "de", res)
        return DiscourseElement(cur_g, self, res, GraphEntity.iri_discourse_element,
                                resp_agent, source, count, label, "de",
                                preexisting_graph)

    def add_id(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> Identifier:
        if res is not None and get_short_name(res) != "id":
            raise ValueError(f"Given res: <{res}> is inappropriate for an Identifier entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_id, "id", res)
        return Identifier(cur_g, self, res, GraphEntity.iri_identifier,
                          resp_agent, source, count, label, "id",
                          preexisting_graph)

    def add_pl(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> PointerList:
        if res is not None and get_short_name(res) != "pl":
            raise ValueError(f"Given res: <{res}> is inappropriate for a PointerList entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_pl, "pl", res)
        return PointerList(cur_g, self, res, GraphEntity.iri_singleloc_pointer_list,
                           resp_agent, source, count, label, "pl",
                           preexisting_graph)

    def add_rp(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> ReferencePointer:
        if res is not None and get_short_name(res) != "rp":
            raise ValueError(f"Given res: <{res}> is inappropriate for a ReferencePointer entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_rp, "rp", res)
        return ReferencePointer(cur_g, self, res, GraphEntity.iri_intextref_pointer,
                                resp_agent, source, count, label, "rp",
                                preexisting_graph)

    def add_ra(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> ResponsibleAgent:
        if res is not None and get_short_name(res) != "ra":
            raise ValueError(f"Given res: <{res}> is inappropriate for a ResponsibleAgent entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_ra, "ra", res)
        return ResponsibleAgent(cur_g, self, res, GraphEntity.iri_agent,
                                resp_agent, source, count, label, "ra",
                                preexisting_graph)

    def add_re(self, resp_agent: str, source: str = None, res: URIRef = None,
               preexisting_graph: Graph = None) -> ResourceEmbodiment:
        if res is not None and get_short_name(res) != "re":
            raise ValueError(f"Given res: <{res}> is inappropriate for a ResourceEmbodiment entity.")
        if res is not None and res in self.res_to_entity:
            return self.res_to_entity[res]
        cur_g, count, label = self._add(self.g_re, "re", res)
        return ResourceEmbodiment(cur_g, self, res, GraphEntity.iri_manifestation,
                                  resp_agent, source, count, label, "re",
                                  preexisting_graph)

    def _add(self, graph_url: str, short_name: str, res: URIRef = None) -> Tuple[Graph, Optional[str], Optional[str]]:
        cur_g: Graph = Graph(identifier=graph_url)
        self._set_ns(cur_g)

        count: Optional[str] = None
        label: Optional[str] = None
        supplier_prefix = get_prefix(res) if res is not None else self.supplier_prefix
        if res is not None:
            try:
                res_count: int = int(get_count(res))
            except ValueError:
                res_count: int = -1
            if res_count > self.counter_handler.read_counter(short_name, supplier_prefix=supplier_prefix):
                self.counter_handler.set_counter(res_count, short_name, supplier_prefix=supplier_prefix)
            return cur_g, count, label

        count = supplier_prefix + str(self.counter_handler.increment_counter(short_name, supplier_prefix=supplier_prefix))

        if self.wanted_label:
            label = "%s %s [%s/%s]" % (self.labels[short_name], count, short_name, count)

        return cur_g, count, label

    def get_orphans(self) -> List[GraphEntity]:
        full_set_of_entities: Set[URIRef] = set(self.res_to_entity.keys())
        referenced_entities: Set[URIRef] = set()
        for res, entity in self.res_to_entity.items():
            for obj in entity.g.objects(subject=res, predicate=None):
                if type(obj) == URIRef:
                    referenced_entities.add(obj)
        set_of_orphan_res: Set[URIRef] = full_set_of_entities - referenced_entities

        result_list: List[GraphEntity] = []
        for orphan_res in set_of_orphan_res:
            entity: Optional[GraphEntity] = self.get_entity(orphan_res)
            if entity is not None:
                result_list.append(entity)

        return result_list

    def remove_orphans_from_triplestore(self, ts_url: str, resp_agent: str) -> None:
        sparql: SPARQLWrapper = SPARQLWrapper(ts_url)

        for entity_res, entity in self.res_to_entity.items():
            if entity.to_be_deleted:
                query: str = f"CONSTRUCT {{?s ?p ?o}} WHERE {{?s ?p ?o ; ?p_1 <{entity_res}>}}"
                sparql.setQuery(query)
                sparql.setMethod('GET')
                sparql.setReturnFormat(RDFXML)

                result: ConjunctiveGraph = sparql.query().convert()
                if result is not None:
                    imported_entities: List[GraphEntity] = Reader.import_entities_from_graph(self, result, resp_agent)
                    for imported_entity in imported_entities:
                        imported_entity.g.remove((imported_entity.res, None, entity_res))

    def commit_changes(self):
        for res, entity in self.res_to_entity.items():
            entity.commit_changes()
            if entity.to_be_deleted:
                del self.res_to_entity[res]

    def _set_ns(self, g: Graph) -> None:
        g.namespace_manager.bind("an", Namespace(self.g_an))
        g.namespace_manager.bind("ar", Namespace(self.g_ar))
        g.namespace_manager.bind("be", Namespace(self.g_be))
        g.namespace_manager.bind("ci", Namespace(self.g_ci))
        g.namespace_manager.bind("de", Namespace(self.g_de))
        g.namespace_manager.bind("br", Namespace(self.g_br))
        g.namespace_manager.bind("id", Namespace(self.g_id))
        g.namespace_manager.bind("pl", Namespace(self.g_pl))
        g.namespace_manager.bind("ra", Namespace(self.g_ra))
        g.namespace_manager.bind("re", Namespace(self.g_re))
        g.namespace_manager.bind("rp", Namespace(self.g_rp))
        g.namespace_manager.bind("biro", GraphEntity.BIRO)
        g.namespace_manager.bind("co", GraphEntity.CO)
        g.namespace_manager.bind("c4o", GraphEntity.C4O)
        g.namespace_manager.bind("cito", GraphEntity.CITO)
        g.namespace_manager.bind("datacite", GraphEntity.DATACITE)
        g.namespace_manager.bind("dcterms", GraphEntity.DCTERMS)
        g.namespace_manager.bind("deo", GraphEntity.DEO)
        g.namespace_manager.bind("doco", GraphEntity.DOCO)
        g.namespace_manager.bind("fabio", GraphEntity.FABIO)
        g.namespace_manager.bind("foaf", GraphEntity.FOAF)
        g.namespace_manager.bind("frbr", GraphEntity.FRBR)
        g.namespace_manager.bind("literal", GraphEntity.LITERAL)
        g.namespace_manager.bind("oa", GraphEntity.OA)
        g.namespace_manager.bind("oco", GraphEntity.OCO)
        g.namespace_manager.bind("prism", GraphEntity.PRISM)
        g.namespace_manager.bind("pro", GraphEntity.PRO)

    def get_an(self) -> Tuple[ReferenceAnnotation]:
        result: Tuple[ReferenceAnnotation] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, ReferenceAnnotation):
                result += (entity, )
        return result

    def get_ar(self) -> Tuple[AgentRole]:
        result: Tuple[AgentRole] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, AgentRole):
                result += (entity, )
        return result

    def get_be(self) -> Tuple[BibliographicReference]:
        result: Tuple[BibliographicReference] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, BibliographicReference):
                result += (entity, )
        return result

    def get_br(self) -> Tuple[BibliographicResource]:
        result: Tuple[BibliographicResource] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, BibliographicResource):
                result += (entity, )
        return result

    def get_ci(self) -> Tuple[Citation]:
        result: Tuple[Citation] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, Citation):
                result += (entity, )
        return result

    def get_de(self) -> Tuple[DiscourseElement]:
        result: Tuple[DiscourseElement] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, DiscourseElement):
                result += (entity, )
        return result

    def get_id(self) -> Tuple[Identifier]:
        result: Tuple[Identifier] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, Identifier):
                result += (entity, )
        return result

    def get_pl(self) -> Tuple[PointerList]:
        result: Tuple[PointerList] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, PointerList):
                result += (entity, )
        return result

    def get_rp(self) -> Tuple[ReferencePointer]:
        result: Tuple[ReferencePointer] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, ReferencePointer):
                result += (entity, )
        return result

    def get_ra(self) -> Tuple[ResponsibleAgent]:
        result: Tuple[ResponsibleAgent] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, ResponsibleAgent):
                result += (entity, )
        return result

    def get_re(self) -> Tuple[ResourceEmbodiment]:
        result: Tuple[ResourceEmbodiment] = tuple()
        for ref in self.res_to_entity:
            entity: GraphEntity = self.res_to_entity[ref]
            if isinstance(entity, ResourceEmbodiment):
                result += (entity, )
        return result
